/*****************************************************************************\
 * Copyright (c) 2004 Mark Aylett <mark@emantic.co.uk>                       *
 *                                                                           *
 * Permission is hereby granted, free of charge, to any person obtaining a   *
 * copy of this software and associated documentation files (the             *
 * "Software"), to deal in the Software without restriction, including       *
 * without limitation the rights to use, copy, modify, merge, publish,       *
 * distribute, sublicense, and/or sell copies of the Software, and to permit *
 * persons to whom the Software is furnished to do so, subject to the        *
 * following conditions:                                                     *
 *                                                                           *
 * The above copyright notice and this permission notice shall be included   *
 * in all copies or substantial portions of the Software.                    *
 *                                                                           *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN *
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,  *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR     *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE *
 * USE OR OTHER DEALINGS IN THE SOFTWARE.                                    *
\*****************************************************************************/

/**
 * \file mar_types_c.h
 * \brief External type definitions.
 */

#ifndef INCLUDED_MAR_TYPES_C
#define INCLUDED_MAR_TYPES_C

#ifndef INCLUDED_FCNTL
#define INCLUDED_FCNTL
#include <fcntl.h>
#endif /* INCLUDED_FCNTL */

#ifndef INCLUDED_STDDEF
#define INCLUDED_STDDEF
#include <stddef.h>
#endif /* INCLUDED_STDDEF */

#ifndef INCLUDED_SYS_TYPES
#define INCLUDED_SYS_TYPES
#include <sys/types.h>
#endif /* INCLUDED_SYS_TYPES */

#if defined(WIN32)
typedef int mode_t;
typedef int ssize_t;
#endif /* WIN32 */

/**
 * \brief The meta key type.
 */

typedef char mar_key_t[38];

/**
 * \brief The meta pair type.
 */

typedef struct mar_pair {
    /**
     * The meta data key.
     */
    mar_key_t key_;
    /**
     * A pointer to the meta data.
     */
    const void* data_;
    /**
     * The size of the meta data.
     */
    size_t size_;
} mar_pair_t;

/**
 * Return code used by some functions to indicate that no matching item could
 * be found.
 */
#define MAR_NOMATCH (-2)

/**
 * \defgroup OpenFlags Open Flags
 * \{
 */

/**
 * Open for reading only.
 */
#define MAR_RDONLY O_RDONLY

/**
 * Open for writing only.
 */
#define MAR_WRONLY O_WRONLY

/**
 * Open for reading and writing.
 */
#define MAR_RDWR O_RDWR

/**
 * Append on each write.
 */
#define MAR_APPEND O_APPEND

/**
 * Create file if it does not exist.
 */
#define MAR_CREAT O_CREAT

/**
 * Truncate size to zero.
 */
#define MAR_TRUNC O_TRUNC

/**
 * Error if create and file exists.
 */
#define MAR_EXCL O_EXCL
/* \} */

/**
 * \defgroup SeekWhence Seek Whence
 * \{
 */

/**
 * The offset is set to offset bytes.
 */
#define MAR_SET 0

/**
 * The offset is set to its current location plus offset bytes.
 */
#define MAR_CUR 1

/**
 * The offset is set to the size of the file plus offset bytes.
 */
#define MAR_END 2
/* \} */

#endif /* INCLUDED_MAR_TYPES_C */

