# Copyright (c) 2004 Mark Aylett <mark@emantic_c.co.uk>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

CL = cl
LINK = link
MKDIR = mkdir
SRCDIR = ..\src

!IF "$(CFG)" == "marl1"

BUILD = release
RUNTIME = ML
TYPE = static

!ELSEIF "$(CFG)" == "marld1"

BUILD = debug
RUNTIME = MLd
TYPE = static

!ELSEIF "$(CFG)" == "mart1"

BUILD = release
RUNTIME = MT
TYPE = static

!ELSEIF "$(CFG)" == "martd1"

BUILD = debug
RUNTIME = MTd
TYPE = static

!ELSEIF "$(CFG)" == "mard1"

BUILD = release
RUNTIME = MT
TYPE = shared

!ELSEIF "$(CFG)" == "mardd1"

BUILD = debug
RUNTIME = MTd
TYPE = shared

!ELSE

!ERROR Configuration either invalid or not specified.

!ENDIF

CDEFS = /D "WIN32" /D "_MBCS"
CFLAGS = /nologo /$(RUNTIME) /W3 /I "..\include" /Fo"$(OUTDIR)\\" /Fd"$(OUTDIR)\\" /FD /c
CXXDEFS = $(CDEFS)
CXXFLAGS = $(CFLAGS) /GX
LDFLAGS = /nologo /incremental:no /machine:I386

OUTDIR = $(CFG)
OUTNAME = $(OUTDIR)\$(CFG)
PROG = $(OUTDIR)\mar.exe
TEST = $(OUTDIR)\mar_test.exe

!IF "$(BUILD)" == "release"

CDEFS = $(CDEFS) /D "NDEBUG"
CFLAGS = $(CFLAGS) /O2

!ELSEIF "$(BUILD)" == "debug"

CDEFS = $(CDEFS) /D "_DEBUG"
CFLAGS = $(CFLAGS) /Od /Zi
LDFLAGS = $(LDFLAGS) /debug /pdbtype:sept

!ENDIF

LDPROG = $(LDFLAGS) /subsystem:console /pdb:"$(OUTDIR)\mar.pdb"

!IF "$(TYPE)" == "static"

CDEFS = $(CDEFS) /D "MAR_STATIC"
CLIB = /D "_LIB"

LDLIB = /nologo
LIBCMD = $(LINK) -lib
LIBMAR = $(OUTNAME).lib

!ELSEIF "$(TYPE)" == "shared"

CDEFS = $(CDEFS) /D "MAR_SHARED"
CLIB = /D "_WINDOWS" /D "_USRDLL"

LDLIB = $(LDFLAGS) /dll /pdb:"$(OUTNAME).pdb" /implib:"$(OUTNAME).lib"
LIBCMD = $(LINK)
LIBMAR = $(OUTNAME).dll

!ENDIF

LIBOBJS = \
	$(OUTDIR)\mar_c.obj \
	$(OUTDIR)\mar_debug_c.obj \
	$(OUTDIR)\mar_defs_c.obj \
	$(OUTDIR)\mar_file_c.obj \
	$(OUTDIR)\mar_flags_c.obj \
	$(OUTDIR)\mar_format_c.obj \
	$(OUTDIR)\mar_info_c.obj \
	$(OUTDIR)\mar_meta_c.obj \
	$(OUTDIR)\mar_mfile_c.obj \
	$(OUTDIR)\mar_mmap_c.obj \
	$(OUTDIR)\mar_seq_c.obj \
	$(OUTDIR)\mar_storage_c.obj \
	$(OUTDIR)\mar_string_c.obj \
	$(OUTDIR)\mar_types_c.obj \
	$(OUTDIR)\mar_user_c.obj \
	$(OUTDIR)\mar_windows_c.obj
PROGOBJS = \
	$(OUTDIR)\getopt.obj \
	$(OUTDIR)\mar_main_c.obj \
	$(OUTDIR)\mar_util_c.obj
TESTOBJS = \
	$(OUTDIR)\mar_test_cpp.obj \

all: $(PROG)

check: $(TEST)
	$(TEST)

clean:
    -@erase /Q $(OUTDIR)\*.*

$(PROG): $(LIBMAR) $(PROGOBJS)
	$(LINK) $(LDPROG) /out:"$(PROG)" $(OUTNAME).lib $(PROGOBJS)

$(TEST): $(LIBMAR) $(TESTOBJS)
	$(LINK) $(LDPROG) /out:"$(TEST)" $(OUTNAME).lib $(TESTOBJS)

$(LIBMAR): $(OUTDIR) $(LIBOBJS)
    $(LIBCMD) $(LDLIB) /out:"$(LIBMAR)" $(LIBOBJS)

$(OUTDIR):
    if not exist $(OUTDIR) $(MKDIR) $(OUTDIR)

$(OUTDIR)\getopt.obj: $(SRCDIR)\getopt.c
    $(CL) $(CFLAGS) $(CDEFS) /D "_CONSOLE" $(SRCDIR)\getopt.c

$(OUTDIR)\mar_main_c.obj: $(SRCDIR)\mar_main_c.c
    $(CL) $(CFLAGS) $(CDEFS) /D "_CONSOLE" $(SRCDIR)\mar_main_c.c

$(OUTDIR)\mar_util_c.obj: $(SRCDIR)\mar_util_c.c
    $(CL) $(CFLAGS) $(CDEFS) /D "_CONSOLE" $(SRCDIR)\mar_util_c.c

$(OUTDIR)\mar_test_cpp.obj: $(SRCDIR)\mar_test_cpp.cpp
    $(CL) $(CXXFLAGS) $(CXXDEFS) /D "_CONSOLE" $(SRCDIR)\mar_test_cpp.cpp

{$(SRCDIR)}.c{$(OUTDIR)}.obj:
    $(CL) $(CFLAGS) $(CDEFS) $(CLIB) /D "MAR_BUILD" $<

