/*****************************************************************************\
 * Copyright (c) 2004 Mark Aylett <mark@emantic.co.uk>                       *
 *                                                                           *
 * Permission is hereby granted, free of charge, to any person obtaining a   *
 * copy of this software and associated documentation files (the             *
 * "Software"), to deal in the Software without restriction, including       *
 * without limitation the rights to use, copy, modify, merge, publish,       *
 * distribute, sublicense, and/or sell copies of the Software, and to permit *
 * persons to whom the Software is furnished to do so, subject to the        *
 * following conditions:                                                     *
 *                                                                           *
 * The above copyright notice and this permission notice shall be included   *
 * in all copies or substantial portions of the Software.                    *
 *                                                                           *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN *
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,  *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR     *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE *
 * USE OR OTHER DEALINGS IN THE SOFTWARE.                                    *
\*****************************************************************************/

static const char rcsid[] = "$Id: mar_info_c.c,v 1.4 2004/12/14 13:09:23 marayl Exp $";

#include "mar_info_c.h"

#include "mar_defs_c.h"
#include "mar_format_c.h"

#include <assert.h>

MAR_EXTERN int
mar_info_(mar_seq_t seq, size_t isize,
          mar_info_t info)
{
    mar_byte_t* addr;
    mar_isize_t local;
    assert(seq && info);

    if (-1 == mar_setregion_(seq, 0, MAR_UDATA(isize)))
        return -1;

    if (!(addr = (mar_byte_t*)mar_seqaddr_(seq)))
        return -1;

    /**
     * Verify the integrity of the advised index size, if incorrect then
     * adjust according to correct value.
     */
    local = mar_decode16_(addr + MAR_ISIZE);
    if (local != isize)
        return mar_info_(seq, local, info);

    info->verno_ = mar_decode16_(addr + MAR_VERNO);
    info->isize_ = local;
    info->index_ = addr + MAR_INDEX;
    info->usize_ = mar_decode32_(addr + MAR_USIZE(local));
    return 0;
}
